@@ -158,8 +158,14 @@ Journalctl_init(Journalctl *self, PyObject *args, PyObject *keywds)
     Py_BEGIN_ALLOW_THREADS
     r = sd_journal_open(&self->j, flags);
     Py_END_ALLOW_THREADS
-    if (r < 0) {
-        PyErr_SetString(PyExc_IOError, "Error opening journal");
+    if (r == -EINVAL) {
+        PyErr_SetString(PyExc_ValueError, "Invalid flags");
+        return -1;
+    }else if (r == -ENOMEM) {
+        PyErr_SetString(PyExc_MemoryError, "Not enough memory");
+        return 1;
+    }else if (r < 0) {
+        PyErr_SetString(PyExc_RuntimeError, "Error opening journal");
         return 1;
     }
 
@@ -236,7 +242,7 @@ Journalctl_get_next(Journalctl *self, PyObject *args)
     }
 
     if (r < 0) {
-        PyErr_SetString(PyExc_IOError, "Error getting next message");
+        PyErr_SetString(PyExc_RuntimeError, "Error getting next message");
         return NULL;
     }else if ( r == 0) { //EOF
         return PyDict_New();
@@ -377,8 +383,16 @@ Journalctl_add_match(Journalctl *self, PyObject *args)
         match_len = match_key_len;
     }
 
-    if (sd_journal_add_match(self->j, match, match_len) != 0) {
-        PyErr_SetString(PyExc_IOError, "Error adding match");
+    int r;
+    r = sd_journal_add_match(self->j, match, match_len);
+    if (r == -EINVAL) {
+        PyErr_SetString(PyExc_ValueError, "Invalid match");
+        return NULL;
+    }else if (r == -ENOMEM) {
+        PyErr_SetString(PyExc_MemoryError, "Not enough memory");
+        return NULL;
+    }else if (r < 0) {
+        PyErr_SetString(PyExc_RuntimeError, "Error adding match");
         return NULL;
     }
 
@@ -433,8 +447,13 @@ PyDoc_STRVAR(Journalctl_add_disjunction__doc__,
 static PyObject *
 Journalctl_add_disjunction(Journalctl *self, PyObject *args)
 {
-    if (sd_journal_add_disjunction(self->j) != 0) {
-        PyErr_SetString(PyExc_IOError, "Error adding disjunction");
+    int r;
+    r = sd_journal_add_disjunction(self->j);
+    if (r == -ENOMEM) {
+        PyErr_SetString(PyExc_MemoryError, "Not enough memory");
+        return NULL;
+    }else if (r < 0) {
+        PyErr_SetString(PyExc_RuntimeError, "Error adding disjunction");
         return NULL;
     }
     Py_RETURN_NONE;
@@ -446,10 +465,6 @@ PyDoc_STRVAR(Journalctl_flush_matches__doc__,
 static PyObject *
 Journalctl_flush_matches(Journalctl *self, PyObject *args)
 {
-    //if (sd_journal_flush_matches(self->j) != 0) {
-    //    PyErr_SetString(PyExc_IOError, "Error flushing matches");
-    //    return NULL;
-    //}
     sd_journal_flush_matches(self->j);
     Py_RETURN_NONE;
 }
@@ -477,8 +492,8 @@ Journalctl_seek(Journalctl *self, PyObject *args, PyObject *keywds)
         Py_BEGIN_ALLOW_THREADS
         r = sd_journal_seek_head(self->j);
         Py_END_ALLOW_THREADS
-        if (r != 0) {
-            PyErr_SetString(PyExc_IOError, "Error seeking to head");
+        if (r < 0) {
+            PyErr_SetString(PyExc_RuntimeError, "Error seeking to head");
             return NULL;
         }
         if (offset > 0LL) {
@@ -495,8 +510,8 @@ Journalctl_seek(Journalctl *self, PyObject *args, PyObject *keywds)
         Py_BEGIN_ALLOW_THREADS
         r = sd_journal_seek_tail(self->j);
         Py_END_ALLOW_THREADS
-        if (r != 0) {
-            PyErr_SetString(PyExc_IOError, "Error seeking to tail");
+        if (r < 0) {
+            PyErr_SetString(PyExc_RuntimeError, "Error seeking to tail");
             return NULL;
         }
         arg = Py_BuildValue("(L)", -1LL);
@@ -508,7 +523,7 @@ Journalctl_seek(Journalctl *self, PyObject *args, PyObject *keywds)
             Py_DECREF(arg);
         }
     }else{
-        PyErr_SetString(PyExc_IOError, "Invalid value for whence");
+        PyErr_SetString(PyExc_ValueError, "Invalid value for whence");
         return NULL;
     }
     Py_RETURN_NONE;
@@ -557,8 +572,8 @@ Journalctl_seek_realtime(Journalctl *self, PyObject *args)
     Py_BEGIN_ALLOW_THREADS
     r = sd_journal_seek_realtime_usec(self->j, timestamp);
     Py_END_ALLOW_THREADS
-    if (r != 0) {
-        PyErr_SetString(PyExc_IOError, "Error seek to time");
+    if (r < 0) {
+        PyErr_SetString(PyExc_RuntimeError, "Error seek to time");
         return NULL;
     }
     Py_RETURN_NONE;
@@ -599,24 +614,32 @@ Journalctl_seek_monotonic(Journalctl *self, PyObject *args)
     }
 
     sd_id128_t sd_id;
+    int r;
     if (bootid) {
-        if (sd_id128_from_string(bootid, &sd_id) != 0) {
+        r = sd_id128_from_string(bootid, &sd_id);
+        if (r == -EINVAL) {
             PyErr_SetString(PyExc_ValueError, "Invalid bootid");
             return NULL;
+        } else if (r < 0) {
+            PyErr_SetString(PyExc_RuntimeError, "Error processing bootid");
+            return NULL;
         }
     }else{
-        if (sd_id128_get_boot(&sd_id) != 0) {
+        r = sd_id128_get_boot(&sd_id);
+        if (r == -EIO) {
             PyErr_SetString(PyExc_IOError, "Error getting current boot ID");
             return NULL;
+        } else if (r < 0) {
+            PyErr_SetString(PyExc_RuntimeError, "Error getting current boot ID");
+            return NULL;
         }
     }
 
-    int r;
     Py_BEGIN_ALLOW_THREADS
     r = sd_journal_seek_monotonic_usec(self->j, sd_id, timestamp);
     Py_END_ALLOW_THREADS
-    if (r != 0) {
-        PyErr_SetString(PyExc_IOError, "Error seek to time");
+    if (r < 0) {
+        PyErr_SetString(PyExc_RuntimeError, "Error seek to time");
         return NULL;
     }
     Py_RETURN_NONE;
@@ -669,8 +692,14 @@ Journalctl_seek_cursor(Journalctl *self, PyObject *args)
     Py_BEGIN_ALLOW_THREADS
     r = sd_journal_seek_cursor(self->j, cursor);
     Py_END_ALLOW_THREADS
-    if (r != 0) {
-        PyErr_SetString(PyExc_IOError, "Error seeking to cursor");
+    if (r == -EINVAL) {
+        PyErr_SetString(PyExc_ValueError, "Invalid cursor");
+        return NULL;
+    }else if (r == -ENOMEM) {
+        PyErr_SetString(PyExc_MemoryError, "Not enough memory");
+        return NULL;
+    }else if (r < 0) {
+        PyErr_SetString(PyExc_RuntimeError, "Error seeking to cursor");
         return NULL;
     }
     Py_RETURN_NONE;
@@ -719,8 +748,14 @@ Journalctl_query_unique(Journalctl *self, PyObject *args)
     Py_BEGIN_ALLOW_THREADS
     r = sd_journal_query_unique(self->j, query);
     Py_END_ALLOW_THREADS
-    if (r != 0) {
-        PyErr_SetString(PyExc_IOError, "Error querying journal");
+    if (r == -EINVAL) {
+        PyErr_SetString(PyExc_ValueError, "Invalid field name");
+        return NULL;
+    } else if (r == -ENOMEM) {
+        PyErr_SetString(PyExc_MemoryError, "Not enough memory");
+        return NULL;
+    } else if (r < 0) {
+        PyErr_SetString(PyExc_RuntimeError, "Error querying journal");
         return NULL;
     }
 
@@ -780,9 +815,14 @@ static PyObject *
 Journalctl_this_boot(Journalctl *self, PyObject *args)
 {
     sd_id128_t sd_id;
-    if (sd_id128_get_boot(&sd_id) != 0) {
+    int r;
+    r = sd_id128_get_boot(&sd_id);
+    if (r == -EIO) {
         PyErr_SetString(PyExc_IOError, "Error getting current boot ID");
         return NULL;
+    } else if (r < 0) {
+        PyErr_SetString(PyExc_RuntimeError, "Error getting current boot ID");
+        return NULL;
     }
 
     char bootid[33];
@@ -792,7 +832,6 @@ Journalctl_this_boot(Journalctl *self, PyObject *args)
     arg = Py_BuildValue("(ss)", "_BOOT_ID", bootid);
     if (! Journalctl_add_match(self, arg)) {
         Py_DECREF(arg);
-        PyErr_SetString(PyExc_IOError, "Error adding match for boot ID");
         return NULL;
     }
     Py_DECREF(arg);
@@ -807,8 +846,13 @@ static PyObject *
 Journalctl_this_machine(Journalctl *self, PyObject *args)
 {
     sd_id128_t sd_id;
-    if (sd_id128_get_machine(&sd_id) != 0) {
-        PyErr_SetString(PyExc_IOError, "Error getting current machine ID");
+    int r;
+    r = sd_id128_get_machine(&sd_id);
+    if (r == -EIO) {
+        PyErr_SetString(PyExc_IOError, "Error getting current boot ID");
+        return NULL;
+    } else if (r < 0) {
+        PyErr_SetString(PyExc_RuntimeError, "Error getting current boot ID");
         return NULL;
     }
 
@@ -819,7 +863,6 @@ Journalctl_this_machine(Journalctl *self, PyObject *args)
     arg = Py_BuildValue("(ss)", "_MACHINE_ID", machineid);
     if (! Journalctl_add_match(self, arg)) {
         Py_DECREF(arg);
-        PyErr_SetString(PyExc_IOError, "Error adding match for machine ID");
         return NULL;
     }
     Py_DECREF(arg);
@@ -827,12 +870,110 @@ Journalctl_this_machine(Journalctl *self, PyObject *args)
     Py_RETURN_NONE;
 }
 
-static PyMemberDef Journalctl_members[] = {
-    {"default_call", T_OBJECT_EX, offsetof(Journalctl, default_call), 0,
-    "default call for values for fields"},
-    {"call_dict", T_OBJECT_EX, offsetof(Journalctl, call_dict), 0,
-    "dictionary of calls for each field"},
-    {NULL}  /* Sentinel */
+static PyObject *
+Journalctl_get_default_call(Journalctl *self, void *closure)
+{
+    Py_INCREF(self->default_call);
+    return self->default_call;
+}
+
+static int
+Journalctl_set_default_call(Journalctl *self, PyObject *value, void *closure)
+{
+    if (value == NULL) {
+        PyErr_SetString(PyExc_TypeError, "Cannot delete default_call");
+        return -1;
+    }
+    if (! PyCallable_Check(value)) {
+        PyErr_SetString(PyExc_TypeError, "default_call must be callable");
+        return -1;
+    }
+    Py_DECREF(self->default_call);
+    Py_INCREF(value);
+    self->default_call = value;
+
+    return 0;
+}
+
+static PyObject *
+Journalctl_get_call_dict(Journalctl *self, void *closure)
+{
+    Py_INCREF(self->call_dict);
+    return self->call_dict;
+}
+
+static int
+Journalctl_set_call_dict(Journalctl *self, PyObject *value, void *closure)
+{
+    if (value == NULL) {
+        PyErr_SetString(PyExc_TypeError, "Cannot delete call_dict");
+        return -1;
+    }
+    if (! PyDict_Check(value)) {
+        PyErr_SetString(PyExc_TypeError, "call_dict must be dict type");
+        return -1;
+    }
+    Py_DECREF(self->call_dict);
+    Py_INCREF(value);
+    self->call_dict = value;
+
+    return 0;
+}
+
+static PyObject *
+Journalctl_get_data_threshold(Journalctl *self, void *closure)
+{
+    size_t cvalue;
+    PyObject *value;
+    int r;
+
+    r = sd_journal_get_data_threshold(self->j, &cvalue);
+    if (r < 0){
+        PyErr_SetString(PyExc_RuntimeError, "Error getting data threshold");
+        return NULL;
+    }
+
+    value = PyInt_FromSize_t(cvalue);
+    return value;
+}
+
+static int
+Journalctl_set_data_threshold(Journalctl *self, PyObject *value, void *closure)
+{
+    if (value == NULL) {
+        PyErr_SetString(PyExc_TypeError, "Cannot delete data threshold");
+        return -1;
+    }
+    if (! PyInt_Check(value)){
+        PyErr_SetString(PyExc_TypeError, "Data threshold must be int");
+        return -1;
+    }
+    int r;
+    r = sd_journal_set_data_threshold(self->j, (size_t) PyInt_AsLong(value));
+    if (r < 0){
+        PyErr_SetString(PyExc_RuntimeError, "Error setting data threshold");
+        return -1;
+    }
+    return 0;
+}
+
+static PyGetSetDef Journalctl_getseters[] = {
+    {"data_threshold",
+    (getter)Journalctl_get_data_threshold,
+    (setter)Journalctl_set_data_threshold,
+    "data threshold",
+    NULL},
+    {"call_dict",
+    (getter)Journalctl_get_call_dict,
+    (setter)Journalctl_set_call_dict,
+    "dictionary of calls for each field",
+    NULL},
+    {"default_call",
+    (getter)Journalctl_get_default_call,
+    (setter)Journalctl_set_default_call,
+    "default call for values for fields",
+    NULL},
+    {NULL}
 };
 
 static PyMethodDef Journalctl_methods[] = {
@@ -900,8 +1041,8 @@ static PyTypeObject JournalctlType = {
     Journalctl_iter,                  /* tp_iter */
     Journalctl_iternext,              /* tp_iternext */
     Journalctl_methods,               /* tp_methods */
-    Journalctl_members,               /* tp_members */
-    0,                                /* tp_getset */
+    0,                                /* tp_members */
+    Journalctl_getseters,             /* tp_getset */
     0,                                /* tp_base */
     0,                                /* tp_dict */
     0,                                /* tp_descr_get */
@@ -953,7 +1094,7 @@ initpyjournalctl(void)
 
     Py_INCREF(&JournalctlType);
     PyModule_AddObject(m, "Journalctl", (PyObject *)&JournalctlType);
-    PyModule_AddStringConstant(m, "__version__", "0.5.0");
+    PyModule_AddStringConstant(m, "__version__", "0.6.0");
     PyModule_AddIntConstant(m, "SD_JOURNAL_NOP", SD_JOURNAL_NOP);
     PyModule_AddIntConstant(m, "SD_JOURNAL_APPEND", SD_JOURNAL_APPEND);
     PyModule_AddIntConstant(m, "SD_JOURNAL_INVALIDATE", SD_JOURNAL_INVALIDATE);
